// iCancelDlg.cpp : implementation file
//

#include "stdafx.h"
#include "iCancel.h"
#include "iCancelDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define DATA_BUFFER_SIZE		4096

const UINT WM_UPDATE_STATUS		= (WM_APP + 1);
const UINT WM_DOWNLOAD_COMPLETE	= (WM_APP + 2);


/////////////////////////////////////////////////////////////////////////////
// CICancelDlg dialog

CICancelDlg::CICancelDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CICancelDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CICancelDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_hSession = NULL;
	m_hRequest = NULL;
	m_pThread = NULL;

	m_dwByteToRead = 0;
	m_dwByteRead = 0;
	m_dwError = ERROR_SUCCESS;

	m_strFileName = _T("");
}

void CICancelDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CICancelDlg)
	DDX_Control(pDX, IDC_DOWNLOAD, m_Download);
	DDX_Control(pDX, IDC_PROGRESS, m_Progress);
	DDX_Control(pDX, IDCANCEL, m_Cancel);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CICancelDlg, CDialog)
	//{{AFX_MSG_MAP(CICancelDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_DOWNLOAD, OnDownload)
	ON_MESSAGE(WM_UPDATE_STATUS, OnUpdateStatus)
	ON_MESSAGE(WM_DOWNLOAD_COMPLETE, OnDownloadComplete)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CICancelDlg message handlers

BOOL CICancelDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// Get a temporary name for the file to download
	TCHAR szPath[MAX_PATH];
	TCHAR szFileName[MAX_PATH];

	GetTempPath(MAX_PATH, szPath);
	GetTempFileName(szPath, _T("ica"), 0, szFileName);

	m_strFileName = szFileName;

	m_Progress.SetRange(0, 100);
	m_Progress.SetPos(0);
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}


void CICancelDlg::Free()
{
	if (m_hRequest)
	{
		InternetCloseHandle(m_hRequest);
		m_hRequest = NULL;
	}

	if (m_hSession)
	{
		InternetCloseHandle(m_hSession);
		m_hSession = NULL;
	}
}
// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CICancelDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CICancelDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CICancelDlg::OnDownload() 
{
	ASSERT(m_pThread == NULL);

	// Trigger the thread that will send the request
	m_pThread = AfxBeginThread(DownloadThread, this);

	if (m_pThread == NULL)
	{
		TRACE0("CICancelDlg::Download: AfxBeginThread failed!\n");
		return;
	}

	m_Download.EnableWindow(FALSE);
	m_Cancel.EnableWindow(TRUE);
}

UINT CICancelDlg::DownloadThread(LPVOID pContext)
{
	CICancelDlg* pDownload = (CICancelDlg*)pContext;
	ASSERT(pDownload);

	pDownload->DownloadFunc();

	return 0;
}

void CICancelDlg::DownloadFunc()
{
	Download(_T("http://noeld.com/download/dynsite.zip"));

	PostMessage(WM_DOWNLOAD_COMPLETE);

	m_pThread = NULL;
}

BOOL CICancelDlg::Download(LPCTSTR pszRequest)
{
	const TCHAR szHeader[] = _T("Accept: */*\r\n\r\n"); // see Q149413
	BOOL bSuccess = FALSE;
	LPBYTE pbBuffer = NULL;
	DWORD dwByteRead;
	DWORD dwSize;
	CFile file;


	m_dwError = ERROR_SUCCESS;

	m_hSession = InternetOpen(_T("iCancel/1.0"), 
							  INTERNET_OPEN_TYPE_PRECONFIG,
							  NULL, NULL, 0);

	if (m_hSession == NULL)
	{
		m_dwError = GetLastError();
		TRACE0("CICancelDlg::Download: Internet session is NULL\n");
		return FALSE;
	}

	m_hRequest = InternetOpenUrl(m_hSession, pszRequest, 
								szHeader, _tcslen(szHeader),
								INTERNET_FLAG_NO_CACHE_WRITE |
								INTERNET_FLAG_PRAGMA_NOCACHE |
								INTERNET_FLAG_RAW_DATA       |
								INTERNET_FLAG_RELOAD, 0);

	if (m_hRequest == NULL)
	{
		m_dwError = GetLastError();
		Free();
		TRACE0("CICancelDlg::Download: InternetOpenUrl failed\n");
		return FALSE;
	}

	// Create the temporary file
	if (!file.Open(m_strFileName, 
				CFile::modeCreate | CFile::typeBinary | 
				CFile::modeWrite | CFile::shareExclusive))
	{
		m_dwError = GetLastError();
		TRACE0("CICancelDlg::Download: could not open temp file!\n");
		Free();
		return FALSE;
	}


	pbBuffer = new BYTE[DATA_BUFFER_SIZE];

	if (NULL == pbBuffer)
	{
		m_dwError = ERROR_NOT_ENOUGH_MEMORY;
		TRACE0("CICancelDlg::Download: not enough memory for pbBuffer!\n");
		goto exit;
	}

	m_dwByteToRead = 0;
	m_dwByteRead = 0;

	dwSize = sizeof(DWORD);

	if (!HttpQueryInfo(m_hRequest, HTTP_QUERY_CONTENT_LENGTH | 
						HTTP_QUERY_FLAG_NUMBER, 
                      (LPVOID)&m_dwByteToRead, &dwSize, NULL))
    {
		m_dwByteToRead = 0;
	}   


	ShowPercent();

	while (TRUE)
	{
		if (!InternetReadFile(m_hRequest, pbBuffer, DATA_BUFFER_SIZE, &dwByteRead))
		{
			m_dwError = GetLastError();

			TRACE0("CICancelDlg::Download: InternetReadFile failed\n");
			break;
		}

		m_dwByteRead += dwByteRead;

		PostMessage(WM_UPDATE_STATUS, dwByteRead);

		if (dwByteRead == 0)
		{
			// If the number of by read is zero then 
			// one reached the end of stream.
			bSuccess = TRUE;
			break;
		}

		try
		{
			file.Write(pbBuffer, dwByteRead);
		}
		catch(...)
		{
			TRACE0("CICancelDlg::Download: unable to write to the file\n");
			break;
		}
	}

exit:

	file.Close();

	// Free the memory
	if (pbBuffer)
		delete [] pbBuffer;

	// Free the handles
	Free();

	return bSuccess;
}

LRESULT CICancelDlg::OnUpdateStatus(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	ShowPercent();

	return 0L;
}

LRESULT CICancelDlg::OnDownloadComplete(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	if (m_dwError == ERROR_SUCCESS)
		AfxMessageBox(IDS_DOWNLOAD_COMPLETE, MB_ICONINFORMATION);
	else
	if (m_dwError == ERROR_INTERNET_OPERATION_CANCELLED)
		AfxMessageBox(IDS_DOWNLOAD_CANCELLED, MB_ICONERROR);
	else
		AfxMessageBox(IDS_DOWNLOAD_FAILED, MB_ICONERROR);

	m_Download.EnableWindow(TRUE);
	m_Cancel.EnableWindow(FALSE);
	m_Progress.SetPos(0);

	// Delete the temporary file
	if (!m_strFileName.IsEmpty())
		DeleteFile(m_strFileName);

	return 0L;
}

DWORD CICancelDlg::GetPercent()
{
	if (m_dwByteToRead == 0)
		return (DWORD)-1;

	return (m_dwByteRead * 100) / m_dwByteToRead;
}

void CICancelDlg::ShowPercent()
{
	int nStat = GetPercent();

	if (nStat > 0)
	{
		m_Progress.SetPos(nStat);
	}
	else
	{
		m_Progress.SetPos(0);
	}
}

void CICancelDlg::OnCancel() 
{
	if (m_hRequest)
	{
		InternetCloseHandle(m_hRequest);
	}

	if (m_hSession)
	{
		InternetCloseHandle(m_hSession);
	}

	m_Cancel.EnableWindow(FALSE);
}

void CICancelDlg::OnOK() 
{
	OnCancel();
	
	CDialog::OnOK();
}

void CICancelDlg::OnDestroy() 
{
	CDialog::OnDestroy();

	Free();
}
